/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename functions or slots use
** Qt Designer which will update this file, preserving your code. Create an
** init() function in place of a constructor, and a destroy() function in
** place of a destructor.
*****************************************************************************/
/***************************************************************************
 *   Copyright (C) 2004 by Reichert Bernard                                *
 *   valen16@trypill.org                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


void AlbumWid::init()
{
	//this will be called on the start
	//set home dir for this user
	m_strHomeDir = QDir::homeDirPath();
	
	//create option tabs
	m_optTab = new QTabWidget(this,"Option Tabs");
	m_optTab->setGeometry( QRect( 356, 140, 330, 290  ) );
	m_sizeWid = new SizeWid(this,"Size");
	m_optTab->addTab(m_sizeWid,"Size");
	m_styleWid = new StyleWid(this,"Style");
	m_optTab->addTab(m_styleWid,"Style");
	m_logoWid = new LogoWid(this,"Logo Blending");
	m_optTab->addTab(m_logoWid,"Logo Blending");
	//set default location:
	m_txLocation->setText(m_strHomeDir);
}


void AlbumWid::destroy()
{
	//this will be called on the end of the programm
	//Clear file list (free some memory)
	onRemoveAll();
}


void AlbumWid::onAddFiles()
{

	//show dialog for file selection and save the selected files into listOfFiles
	QStringList listOfFiles = QFileDialog::getOpenFileNames(
                            "Images (*.png *.xpm *.jpg *.JPG)",
                            m_strHomeDir,
                            this,
                            "Select images to add to your album"
                            "Select images to add to your album" );
			    
	//add files to list
 	m_listFiles->insertStringList(listOfFiles);
	
	//add files into m_vecFileList
	for(uint i = 0;i<listOfFiles.count();i++)
	{
		//new FileItem
		FileItem* pFileIt = new FileItem;
		//set filepath
		pFileIt->fileName = listOfFiles[i];
		//set filename
		pFileIt->rotation = 0;
		//store it in a vector
		m_vecFileList.push_back(pFileIt);
	}
}

void AlbumWid::onRemoveAll()
{
	//run through the vector and delete each item
	for(uint i = 0;i<m_vecFileList.size();i++)
	{
		delete m_vecFileList.at(i);
	}
	//clear vector
	m_vecFileList.clear();
	//clear file list
	m_listFiles->clear();
}


void AlbumWid::onSetLocation()
{
	//set location where album should be created
	
	//show folder dialog
	QString strFolder = QFileDialog::getExistingDirectory(m_strHomeDir,
		this,
		"Choose a directory","",
		TRUE ,FALSE);
		
	//if no / is at the end add it to avoid bugs
	if(strFolder[strFolder.length()-1]!='/')
		strFolder+='/';
	
	//show it in the filed
	m_txLocation->setText(strFolder);
}


void AlbumWid::onRotate(int rotation)
{
	//load path to current image
	QString strImg = m_listFiles->currentText();
	if(strImg!="")
	{
		//run through the vector
		for(uint i = 0;i<m_vecFileList.size();i++)
		{
		
			if(m_vecFileList[i]->fileName == strImg)
			{
				//store rotation in correct vector item
				m_vecFileList[i]->rotation = rotation;
				break;
			}
		}
	}
	if(m_listFiles->selectedItem())
	{
		//reload image
		emit onPreviewImage(m_listFiles->selectedItem());
	}
}


void AlbumWid::onPreviewImage(QListBoxItem* listItem)
{
//show image in the preview box
	//if a rotation was set it will also be rotatet
	
	//load path to current image
	QString strImg = listItem->text();
	//create a rotation variable
	int rotation = 0;
	
	//check rotation in vector list
	for(uint i = 0;i<m_vecFileList.size();i++)
	{
		
		if(m_vecFileList[i]->fileName == strImg)
		{
			rotation = m_vecFileList[i]->rotation;
			break;
		}
	}
	//conversion matrix
	QWMatrix matrix;
	
	//rotate
	switch(rotation)
	{
		case 0:
			//set rotation
			matrix.rotate( 0.0 );
			//set button
			m_butgrpRotation->setButton(m_butgrpRotation->id(radioRot0));
			break;
		case 1:
			matrix.rotate( 90.0 );
			m_butgrpRotation->setButton(m_butgrpRotation->id(radioRot90));
			break;
		case 2:
			matrix.rotate( 180.0 );
			m_butgrpRotation->setButton(m_butgrpRotation->id(radioRot180));
			break;
		case 3:
			matrix.rotate( 270.0 );
			m_butgrpRotation->setButton(m_butgrpRotation->id(radioRot270));
			break;
	}
	//create image
	QPixmap *pix = new QPixmap;
	QImage *img = new QImage;
	
	//if image does not fails to load rotate it
	if(!img->load(strImg)){}
	else
	{
		//rotate
		*img = img->xForm( matrix );
	}
	
	//set image
	pix->convertFromImage(*img);
	m_imgPreview->setPixmap(*pix);
	delete pix;
	delete img;
}

void AlbumWid::onCreateAlbum()
{
	gThread = new GalleryThrd;
	
	//what should be done
	gThread->setBools(
			m_sizeWid->m_ckThumbs->isChecked(),
			m_styleWid->m_ckWebsite->isChecked(),
			(m_logoWid->m_txLogo->text().length()>0)?true:false);
	//set filenames, extensions, style & ...
	if(m_txLocation->text().at(m_txLocation->text().length()-1)!='/')
		m_txLocation->setText(m_txLocation->text()+'/');
	gThread->setStrings(
			m_cbImageType->currentText(),
			m_txLocation->text(),
			m_txPattern->text(),
			m_styleWid->m_txTitle->text(),
			m_logoWid->m_txLogo->text(),
			m_styleWid->m_cbStyle->currentText());
	
	//set diffrent siize properties
	gThread->setIntegers(
			m_logoWid->m_butgrpPlace->selectedId(),
			m_sizeWid->m_txWidthO->text().toInt(),
			m_sizeWid->m_txHeightO->text().toInt(),
			m_sizeWid->m_txWidthT->text().toInt(),
			m_sizeWid->m_txHeightT->text().toInt(),
			m_sizeWid->m_butgrpResizeType->selectedId());

	gThread->setProgressBar(m_ctProgress);
	gThread->setFileList(&m_vecFileList);
	gThread->start();
}
