/***************************************************************************
 *   Copyright (C) 2004 by Reichert Bernard                                *
 *   valen16@trypill.org                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qdir.h> 
#include "GalleryThrd.h"

 
void GalleryThrd::run()
{
	m_nCount = 0;
	
	if(m_bLogo)
	{
		m_imageLogo = imlib_load_image(m_strLogo);
		if(!m_imageLogo)
		{
			m_bLogo=false;
		}
		else
		{
			imlib_context_set_image(m_imageLogo);
			m_logo_height = imlib_image_get_height();
			m_logo_width = imlib_image_get_width();
		}
	}
	//thread started
	//execute duties
	begin();
	//kill thread
	exit();
}
 
void GalleryThrd::setBools(bool bThumbs, bool bSite, bool bLogo)
{
	m_bThumbs = bThumbs;
	m_bSite = bSite;
	m_bLogo = bLogo;
}
 
void GalleryThrd::setStrings(QString strType,QString strLocation,QString strPattern,QString strTitle,QString strLogo,QString strStyle)
{
	m_strTitle = strTitle;
	m_strType = strType;
	m_strLocation = strLocation;
	m_strPattern = strPattern;
	m_strLogo = strLogo;
	m_strStyle = strStyle;
	//reformat extension
	reformatExt();
}
 
void GalleryThrd::setIntegers(int nLogoPos, int owidth, int oheight, int twidth, int theight, int resizeType)
{
	//thumbnail sizes
	m_twidth = twidth;
	m_theight = theight;
	//large image sizes
	m_owidth = owidth;
	m_oheight = oheight;
	//resize type (ratio)
	m_resizeType = resizeType;
	//logo postition..
	m_nLogoPos = nLogoPos;
}
 
void GalleryThrd::setProgressBar(QProgressBar *ctProgress)
{
	m_ctProgress = ctProgress;
}
void GalleryThrd::setFileList(vector<FileItem*> *pvecFileList)
{
	m_pvecFileList= pvecFileList;
}
 
void GalleryThrd::begin()
{
	//image in which we will store the current one
	Imlib_Image image;
	
	uint files = m_pvecFileList->size();
	
	//initialise progress bar
	m_ctProgress->setTotalSteps((int)files);
	
	//if a site should be created, create it
	if(m_bSite)
		doCreateWebsite(files);
	//run through all files
	for(uint i = 0; i<files;i++)
	{
		//try loading image
		image = imlib_load_image_without_cache(m_pvecFileList->at(i)->fileName);
		//if files was loaded:
		if (image)
		{   
			//set the context to this image
			imlib_context_set_image(image);
			//rotate it as needed
			imlib_image_orientate(m_pvecFileList->at(i)->rotation);
			//load sizes of the current image
			orig_height = imlib_image_get_height();
			orig_width = imlib_image_get_width();
			
			//et size of the large images
			recalcLarge();
			
			//copy image to image resized with the 
			Imlib_Image imgresized = imlib_create_cropped_scaled_image(
                                 		0,0,
				 		orig_width,orig_height,
                                 		m_owidth, m_oheight
                                 	); 
			//set the context to this imageresized
			imlib_context_set_image(imgresized);
			
			//if a logo should be created do it
			if(m_bLogo)
				doLogoBlending();
			
			//format number
			QString strNumber = formatNumber(i+m_nCount);

			//create file name for large image
			QString strFile = m_strLocation+m_strPattern+strNumber+"."+m_strExt;
			//set output type (jpeg,pngm,tiff...)
			imlib_image_set_format(m_strType);
			//save image in file strFile
			imlib_save_image(strFile);
			//if thumbnails should be create do it
			if(m_bThumbs)
				doThumbnail(&imgresized,strNumber);
			//free resized image
			imlib_free_image();
			//set cintext to image
			imlib_context_set_image(image);
			//free image
			imlib_free_image();
		}
		//increase progres bar value
		m_ctProgress->setProgress(i+1);
	}
}
 
void GalleryThrd::recalcLarge()
{
	//recalculate size for the big image
	if(m_resizeType == 2){//height
		if(orig_height<m_oheight)
		{
		}else{
			double ratio = (double)orig_width/(double)orig_height;
			m_owidth = (int)(m_oheight*ratio);
		}
	}else if(m_resizeType == 1){//width
		if(orig_width<m_owidth)
		{
		}else{
			double ratio = (double)orig_height/(double)orig_width;
			m_oheight = (int)(m_owidth*ratio);
		}
	}
}

void GalleryThrd::reformatExt()
{
	//format extension to avoid bugs in other OS's
	if(m_strType == "jpeg")
		m_strExt = "jpg";
	else
		m_strExt = m_strType;
}

QString GalleryThrd::formatNumber(int number)
{
	//format number for image names
	QString strNumber;
	if(number<10)
	{
		strNumber = "000"+QString::number(number,10);
	}else if(number<100)
	{
		strNumber = "00"+QString::number(number,10);
	}
	else if(number<1000)
	{
		strNumber = "0"+QString::number(number,10);
	}
	return strNumber;
}
void GalleryThrd::doThumbnail(Imlib_Image *imgresized,QString strNumber)
{
	int height = m_theight;
	int width = m_twidth;
	//recalculate size
	if(m_resizeType == 2){//resize to height
		if(orig_height<m_oheight)
		{
		}else{
			double ratio = (double)m_owidth/(double)m_oheight;
			width = (int)(height*ratio);
		}
	}else if(m_resizeType == 1){//resize to width
		if(orig_width<m_owidth)
		{
		}else{
			double ratio = (double)orig_height/(double)m_owidth;
			height = (int)(width*ratio);
		}
	}
	
	//create new resized image
	Imlib_Image imgfinal = imlib_create_cropped_scaled_image(
			0,0,
			m_owidth,m_oheight,
			width, height
			); 

	imlib_context_set_image(imgfinal);
	
	//set new filename
	QString strFile = m_strLocation+"/"+m_strPattern+strNumber+"_thumb"+"."+m_strExt;

	//save thumbnail
	imlib_save_image(strFile);
	//free thumbnail
	imlib_free_image();
	//set context back to original image
	imlib_context_set_image(*(Imlib_Image*)imgresized);
}

void GalleryThrd::doLogoBlending()
{
	imlib_context_set_blend(1);
	//copy logo onto the bla..
	if(m_imageLogo)
	{
		switch(m_nLogoPos)
		{
			case 0:
				imlib_blend_image_onto_image(m_imageLogo,0,0,0,m_logo_width,m_logo_height,0,0,m_logo_width,m_logo_height);
				break;
			case 1:
				imlib_blend_image_onto_image(m_imageLogo,0,0,0,m_logo_width,m_logo_height,m_owidth-m_logo_width,0,m_logo_width,m_logo_height);
				break;
			case 2:
				imlib_blend_image_onto_image(m_imageLogo,0,0,0,m_logo_width,m_logo_height,0,m_oheight-m_logo_height,m_logo_width,m_logo_height);
				break;
			case 3:
				imlib_blend_image_onto_image(m_imageLogo,0,0,0,m_logo_width,m_logo_height,m_owidth-m_logo_width,m_oheight-m_logo_height,m_logo_width,m_logo_height);
				break;
			case 4:
				imlib_blend_image_onto_image(m_imageLogo,0,0,0,m_logo_width,m_logo_height,(m_owidth/2)-(m_logo_width/2),(m_oheight/2)-(m_logo_height/2),m_logo_width,m_logo_height);
				break;
		}
	}
}

void GalleryThrd::doCreateWebsite(uint files)
{
	//create configuration file
	QFile* pFile2 = new QFile(m_strLocation+"/config.js");
	pFile2->open(IO_Append|IO_WriteOnly);
	//configuration content
	QString strConfigJS=
	"////javascript\n"
	"//contact: valen16@trypill.org\n"
	"//kphotools     --        2004\n"
	"//configuration file\n"
	"//base image name\n"
	"var base_name=\""+m_strPattern+"\";\n"
	"//extension\n"
	"var type = \""+m_strExt+"\";\n"
	"//number of images\n"
	"var img_count = "+QString::number(files)+";\n"
	"//page title\n"
	"var page_title=\""+this->m_strTitle+"\";\n";
	
	//write
	QTextStream stream( pFile2 );
		stream << strConfigJS << "\n";
	pFile2->close();
	
	//read style configuration
	QStringList strFiles = readConfig();
	
	//for each line
	for(uint i = 0;i<strFiles.count();i++)
	{
		if(strFiles[i][0] == 'f')//if is file copy it
		{
			strFiles[i].remove(0,1);
			system("cp /usr/share/kphotools/styles/"+m_strStyle+"/"+strFiles[i]+"  "+m_strLocation+"/"+strFiles[i]);
		}else if(strFiles[i][0] == 'd')//if is dir create it
		{
			strFiles[i].remove(0,1);
			QDir* newDir  = new QDir();
			m_strLocation+=strFiles[i]+"/";
			newDir->mkdir(m_strLocation);
		}
	}
	
}
QStringList GalleryThrd::readConfig()
{
	//load configuration for style
	QFile* pFile = new QFile("/usr/share/kphotools/styles/"+m_strStyle+"/config");
	pFile->open(IO_ReadOnly);
	QStringList strList;
	QString strLine = "";
	//read it
	while(!pFile->atEnd())
	{
		//read each line
		pFile->readLine(strLine,255);
		strLine.replace("\n","");
		if(strLine[0]!='#')
			strList+=strLine;
	}
	pFile->close();
	delete pFile;
	//return config
	return strList;
}
