/***************************************************************************
 *   Copyright (C) 2004 by Reichert Bernard                                *
 *   valen16@trypill.org                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

 #include <qthread.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <Imlib2.h>

 //class for previews and taking screenshots
 class ScreenshotThrd : public QThread
 {
 	private:
		//Display which will be captured
		Display *m_pDisp;
		//other display settings
		Visual *m_pVis;
		Colormap m_cm;
		
		//file to stroe screenshot
		QString m_strFile;
		
		//color depht
		unsigned int m_nDepth;
		
		//screensize
		uint m_screen_width;
		uint m_screen_height;
		//time until screenshot will be taken
		int m_nTimeout;
		
		//screen number
		int m_nScreen;
		
		//sizes
		int m_w,m_h;
		int m_nx1,m_nx2,m_ny1,m_ny2;
		
		//take complete window?
		bool completewindow;
			
	public:
		ScreenshotThrd():QThread()
		{
			//fetch complete window = no
			completewindow = false;
			
			//set sizes
			m_w = 0;
			m_h = 0;
			m_nx1 = 0;
			m_nx2 = 0;
			m_ny1 = 0;
			m_ny2 = 0;
			
			//get display information
			m_pDisp = XOpenDisplay(0);
			m_nScreen = DefaultScreen(m_pDisp);
			m_screen_width = DisplayWidth(m_pDisp, m_nScreen);
			m_screen_height = DisplayHeight(m_pDisp, m_nScreen);
			m_pVis = DefaultVisual(m_pDisp, m_nScreen);
			m_cm = DefaultColormap(m_pDisp, m_nScreen);
			
			//set imlib properties
			imlib_context_set_display(m_pDisp);
			imlib_context_set_visual(m_pVis);
			imlib_context_set_colormap(m_cm);
			imlib_context_set_drawable(RootWindow(m_pDisp, m_nScreen));
			imlib_context_set_anti_alias(1);
			imlib_context_set_blend(0);
		}
		~ScreenshotThrd()
		{
			XCloseDisplay(m_pDisp);
		}
		
		void setSize(int w,int h)
		{
			m_w = w;
			m_h = h;
		}
		
		//set window which should be fetched
		void selectWindow(int x1,int x2,int y1,int y2)
		{
			m_nx1 = x1;
			m_nx2 = x2;
			m_ny1 = y1;
			m_ny2 = y2;
		}
		
		//fetch window?
		void setCompleteWindow(bool bDo)
		{
			completewindow = bDo;
		}
		
		//set timeout befor fetching
		void setTimeout(int nTimeout = 0)
		{
			m_nTimeout = nTimeout;
		}
		//set File where files
		void setFile(QString strFile)
		{
			m_strFile = strFile;
		}
	virtual void run(){
		//wait m_nTimeout milli seconds
		sleep(m_nTimeout);
		//fetch image from screen
		Imlib_Image img = imlib_create_image_from_drawable((Pixmap)0,0,0,m_screen_width,m_screen_height,1);
		
		//set context to the image which was captured
		imlib_context_set_image(img);
		//if positionn was set in small window, fetch it
		if(m_nx1>0 && m_ny1>0 && !completewindow)
		{
			//recalulate size from button size to current original size
			int x1 = (int)((((double)m_nx1)/430.0)*(double)m_screen_width);
			int x2 = (int)((((double)m_nx2)/430.0)*(double)m_screen_width);
			int y1 = (int)((((double)m_ny1)/390.0)*(double)m_screen_height);
			int y2 = (int)((((double)m_ny2)/390.0)*(double)m_screen_height);
			
			//copy screenpart
			Imlib_Image imgfi = imlib_create_cropped_scaled_image(
				x1,y1,
				x2-x1,y2-y1,
				x2-x1,y2-y1 //int destination_x, int destination_y,
				); 	
			//free complete screen
			imlib_free_image();
			//set context to current
			imlib_context_set_image(imgfi);
		}
		else if(m_nx1>0 && m_ny1>0 && completewindow)
		{
			//copy complete window to final image
			Imlib_Image imgfi = imlib_create_cropped_scaled_image(
				m_nx1,m_ny1,
				m_nx2,m_ny2,
				m_nx2,m_ny2 
				); 	
			//free complete screen
			imlib_free_image();
			//set context to current
			imlib_context_set_image(imgfi);
		}
		//if width > 0 and height >0
		if(m_w>0 && m_h>0)
		{
			//resize it to size set
			Imlib_Image imgfi = imlib_create_cropped_scaled_image(0, 0, 
				m_screen_width, m_screen_height,
					m_w,
					m_h); 	
			//free complete screen
			imlib_free_image();
			//set context to current
			imlib_context_set_image(imgfi);
		}
		
		//set output format
		imlib_image_set_format("png");
		//save
		imlib_save_image(m_strFile);
		//free image
		imlib_free_image_and_decache();

		//end thread
		exit();
	}
 };
